/*
 * Copyright 2011 The Kuali Foundation.
 * 
 * Licensed under the Educational Community License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.opensource.org/licenses/ecl2.php
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.kuali.kfs.gl.batch.service;

import org.kuali.kfs.gl.businessobject.SufficientFundRebuild;


/**
 * An interface declaring a method that runs the sufficient funds rebuilder process.
 */
public interface SufficientFundsAccountUpdateService {

    /**
     * Rebuilds all necessary sufficient funds balances.
     */
    public void rebuildSufficientFunds();
    
    /**
     * Given an O SF rebuild type, it will look up all of the matching balances in the table and add each account it finds as an A
     * SF rebuild type.
     * 
     * @param sfrb the sufficient fund rebuild record to convert
     */
    public abstract void convertOtypeToAtypes(SufficientFundRebuild sfrb);
    
    /**
     * Updates sufficient funds balances for the given account
     * 
     * @param sfrb the sufficient fund rebuild record, with a chart and account number
     */
    public abstract void calculateSufficientFundsByAccount(SufficientFundRebuild sfrb);
}
