/*
 * Copyright 2011 The Kuali Foundation.
 * 
 * Licensed under the Educational Community License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.opensource.org/licenses/ecl2.php
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.kuali.kfs.gl.dataaccess;

import java.util.Collection;

import org.kuali.kfs.gl.businessobject.OriginEntrySource;

/**
 * An interface for helping OriginEntrySource objects interact with the database
 */

public interface OriginEntrySourceDao {
    /**
     * Fetches all origin entry full records in the database
     * 
     * @return a Collection of all origin entry source records
     */
    public Collection findAll();

    /**
     * Finds an origin entry source record based on its source code
     * 
     * @param code the code of the origin entry source record to find
     * @return an Origin Entry Source record if found, or null if not found
     * @return
     */
    public OriginEntrySource findBySourceCode(String code);
}
